
// Command controller for NoteTaker commands

var Cc = Components.classes;
var Ci = Components.interfaces;

var keywordController = {
  _cmds : { "notetaker-keyword-add" : true,
            "notetaker-keyword-delete" : true,
            "notetaker-keyword-commit" : true,
            "notetaker-keyword-undo-all" : true
          },
  _undo_stack : [],
  _rdf : null,
  _ds : null,
  _ns : "http://www.mozilla.org/notetaker-rdf#",
  _related : null,
  _label : null,
  _keyword : null,

  init             : function (ds) {
    this._rdf = Cc["@mozilla.org/rdf/rdf-service;1"];
    this._rdf = this._rdf.getService(Ci.nsIRDFService);
    this._ds = ds;
    this._related = this._rdf.GetResource(this._ns + "related");
    this._label   = this._rdf.GetResource(this._ns + "label");
    this._keyword = this._rdf.GetResource(this._ns + "keyword");
    window.controllers.insertControllerAt(0,this);
  },

  _LoggedAssert     : function (sub, pred, obj)
  {
    if ( !this._ds.HasAssertion(sub, pred, obj, true))
    {
alert("pushA");
      this._undo_stack.push( { assert:true, sterm:sub, pterm:pred, oterm:obj } );
      this._ds.Assert(sub, pred, obj, true);
    }
  },

  _LoggedUnassert   : function (sub, pred, obj)
  {
    if ( this._ds.HasAssertion(sub, pred, obj, true))
    {
alert("pushU");
      this._undo_stack.push( { assert:false, sterm:sub, pterm:pred, oterm:obj } );
      this._ds.Unassert(sub, pred, obj, true);
    }
  },

  supportsCommand  : function (cmd) { return (cmd in this._cmds); },
  isCommandEnabled : function (cmd) { return true; },
  onEvent          : function (cmd) { return true; },
  doCommand        : function (cmd) {

    var url     = window.opener.content.document.location.href;
    var keyword = window.document.getElementById("dialog.keyword").value;

    if (keyword.match(/^[ \t]*$/))
      return;

    var keyword_cont = this._rdf.GetResource("urn:notetaker:keywords");
    var keyword_node = this._rdf.GetResource("urn:notetaker:keyword:" + keyword);
    var keyword_value = this._rdf.GetLiteral(keyword);
    var url_node     = this._rdf.GetResource(url);

    var test_node, keyword2, enum1, enum2;

    // Add and Delete cases should also work on the urn:notetaker:keywords
    // <Seq>. Requires an enhancement to the undo system if
    // AppendElement() and RemoveElement() are to be used on that container

    switch (cmd) {
    case "notetaker-keyword-add":
      // This keyword should be related to an existing keyword, if any
      keyword2 = this._ds.GetTarget(url_node, this._keyword, true);
      if (keyword2)
        this._LoggedAssert(keyword_node, this._related, keyword2);

      // add this keyword
      this._LoggedAssert(keyword_node, this._label, keyword_value);

      // add this keyword to the current note.
      this._LoggedAssert(url_node, this._keyword, keyword_node);
      break;

    case "notetaker-keyword-delete":
      // remove this keyword from the current note.
      this._LoggedUnassert(url_node, this._keyword, keyword_node);

      // remove this keyword and related facts if it's not used elsewhere
      enum1 = this._ds.GetSources( this._keyword, keyword_node, true);
      if (!enum1.hasMoreElements())
      {
        // this keyword
        this._LoggedUnassert(keyword_node, this._label, keyword_value);

        // this keyword is related to that keyword
        enum2 = this._ds.GetTargets(keyword_node, this._related, true);
        while (enum2.hasMoreElements())
          this._LoggedUnassert(keyword_node, this._related,
                              enum2.getNext().QueryInterface(Ci.nsIRDFNode));

        // that keyword is related to this keyword
        enum2 = this._ds.GetSources(this._related, keyword_node, true);
        while (enum2.hasMoreElements())
          this._LoggedUnassert(enum2.getNext().QueryInterface(Ci.nsIRDFNode),
                              this._related, keyword_node);
      }
      else // this keyword is used elsewhere.
      {
        // delete related facts where keywords that this keyword
        // relates to are only found in the current note.
        enum1 = this._ds.GetTargets(keyword_node, this._related, true);
        while (enum1.hasMoreElements())
        {
          keyword2 = enum1.getNext().QueryInterface(Ci.nsIRDFNode);
          enum2 = this._ds.GetSources(this._keyword, keyword2, true);

          test_node = enum2.getNext().QueryInterface(Ci.nsIRDFNode);
          if (!enum2.hasMoreElements() && test_node.EqualsNode(url_node))
            this._LoggedUnassert(keyword_node, this._related, keyword2);

          // delete related facts where keyword that relates to this
          // keyword are only found in the current note.
          enum1 = this._ds.GetSources(this._related, keyword_node, true);
          while (enum1.hasMoreElements())
          {
            keyword2 = enum1.getNext().QueryInterface(Ci.nsIRDFNode);
            enum2 = this._ds.GetSources(this._keyword, keyword2, true);

            test_node = enum2.getNext().QueryInterface(Ci.nsIRDFNode);
            if (!enum2.hasMoreElements() && test_node.EqualsNode(url_node))
              this._LoggedUnassert(keyword2, this._related, keyword_node);
          }
        }
      }
      break;

    case "notetaker-keyword-commit":
      this._undo_stack = [];
      break;

    case "notetaker-keyword-undo-all":
      while (this._undo_stack.length > 0 )
      {
        var cmd = this._undo_stack.pop();
        if ( cmd.assert )
          this._ds.Unassert(cmd.sterm, cmd.pterm, cmd.oterm, true);
        else
          this._ds.Assert(cmd.sterm, cmd.pterm, cmd.oterm, true);
      }
      break;

    default:
      throw "Unknown notetaker-keyword- command";
      break;
    }
  }
};

keywordController.init(window.opener.noteSession.datasource);

